// Copyright 2015 The Gogs Authors. All rights reserved.
// Use of this source code is governed by a MIT-style
// license that can be found in the LICENSE file.

package gogs

type MarkdownOption struct {
	Text    string
	Mode    string
	Context string
}
